from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Voltage:
	"""Voltage commands group definition. 17 total commands, 7 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("voltage", core, parent)

	@property
	def ainput(self):
		"""ainput commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_ainput'):
			from .Ainput import Ainput
			self._ainput = Ainput(self._core, self._cmd_group)
		return self._ainput

	@property
	def sense(self):
		"""sense commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_sense'):
			from .Sense import Sense
			self._sense = Sense(self._core, self._cmd_group)
		return self._sense

	@property
	def dvm(self):
		"""dvm commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_dvm'):
			from .Dvm import Dvm
			self._dvm = Dvm(self._core, self._cmd_group)
		return self._dvm

	@property
	def negative(self):
		"""negative commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_negative'):
			from .Negative import Negative
			self._negative = Negative(self._core, self._cmd_group)
		return self._negative

	@property
	def level(self):
		"""level commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_level'):
			from .Level import Level
			self._level = Level(self._core, self._cmd_group)
		return self._level

	@property
	def protection(self):
		"""protection commands group. 0 Sub-classes, 4 commands."""
		if not hasattr(self, '_protection'):
			from .Protection import Protection
			self._protection = Protection(self._core, self._cmd_group)
		return self._protection

	@property
	def ramp(self):
		"""ramp commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_ramp'):
			from .Ramp import Ramp
			self._ramp = Ramp(self._core, self._cmd_group)
		return self._ramp

	def get_range(self) -> float:
		"""SCPI: [SOURce]:VOLTage:RANGe \n
		Snippet: value: float = driver.source.voltage.get_range() \n
		No command help available \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('SOURce:VOLTage:RANGe?')
		return Conversions.str_to_float(response)

	def set_range(self, arg_0: float) -> None:
		"""SCPI: [SOURce]:VOLTage:RANGe \n
		Snippet: driver.source.voltage.set_range(arg_0 = 1.0) \n
		No command help available \n
			:param arg_0: No help available
		"""
		param = Conversions.decimal_value_to_str(arg_0)
		self._core.io.write(f'SOURce:VOLTage:RANGe {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ...Internal.Types import DataType
from ...Internal.ArgSingleList import ArgSingleList
from ...Internal.ArgSingle import ArgSingle
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Duration:
	"""Duration commands group definition. 1 total commands, 0 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("duration", core, parent)

	def set(self, set_new_value: float, return_min_or_max: enums.MinOrMax = None) -> None:
		"""SCPI: LOG:DURation \n
		Snippet: driver.log.duration.set(set_new_value = 1.0, return_min_or_max = enums.MinOrMax.MAX) \n
		Sets or queries the duration of the data logging. \n
			:param set_new_value:
				- numeric value: Duration of the data logging captured in the range of 0 s to 3.49*10^5 s.
				- MIN | MINimum: Minimum duration of the data logging captured at 0 s.
				- MAX | MAXimum: Maximum duration of the data logging captured at 3.49*10^5 s.
			:param return_min_or_max: Returns the duration of the data logging."""
		param = ArgSingleList().compose_cmd_string(ArgSingle('set_new_value', set_new_value, DataType.Float), ArgSingle('return_min_or_max', return_min_or_max, DataType.Enum, enums.MinOrMax, is_optional=True))
		self._core.io.write(f'LOG:DURation {param}'.rstrip())

	def get(self, return_min_or_max: enums.MinOrMax = None) -> int:
		"""SCPI: LOG:DURation \n
		Snippet: value: int = driver.log.duration.get(return_min_or_max = enums.MinOrMax.MAX) \n
		Sets or queries the duration of the data logging. \n
			:param return_min_or_max: Returns the duration of the data logging.
			:return: result: No help available"""
		param = ArgSingleList().compose_cmd_string(ArgSingle('return_min_or_max', return_min_or_max, DataType.Enum, enums.MinOrMax, is_optional=True))
		response = self._core.io.query_str(f'LOG:DURation? {param}'.rstrip())
		return Conversions.str_to_int(response)



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Logic:
	"""Logic commands group definition. 1 total commands, 1 Sub-groups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("logic", core, parent)

	@property
	def dio(self):
		"""dio commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_dio'):
			from .Dio import Dio
			self._dio = Dio(self._core, self._cmd_group)
		return self._dio



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="RsNgx",
    version="3.1.0.49",
    description="NGx Power Supply Remote-control Module",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Rohde & Schwarz GmbH & Co. KG",
    copyright="Copyright © Rohde & Schwarz GmbH & Co. KG 2021",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
    ],
    packages=(find_packages(include=['RsNgx', 'RsNgx.*'])),
    install_requires=['PyVisa']
)


from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SymbolRate:
	"""SymbolRate commands group definition. 1 total commands, 0 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("symbolRate", core, parent)

	def get_state(self) -> bool:
		"""SCPI: OUTPut:SRATe[:STATe] \n
		Snippet: value: bool = driver.output.symbolRate.get_state() \n
		Sets or queries the reduce slew rate option for the selected channel. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str_with_opc('OUTPut:SRATe:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, arg_0: bool) -> None:
		"""SCPI: OUTPut:SRATe[:STATe] \n
		Snippet: driver.output.symbolRate.set_state(arg_0 = False) \n
		Sets or queries the reduce slew rate option for the selected channel. \n
			:param arg_0:
				- 1: Activates reduce slew rate option for the selected channel.
				- 0: Deactivates reduce slew rate option for the selected channel."""
		param = Conversions.bool_to_str(arg_0)
		self._core.io.write_with_opc(f'OUTPut:SRATe:STATe {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Vnc:
	"""Vnc commands group definition. 2 total commands, 0 Sub-groups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("vnc", core, parent)

	def get_state(self) -> bool:
		"""SCPI: SYSTem:VNC:STATe \n
		Snippet: value: bool = driver.system.vnc.get_state() \n
		Enables or disables VNC state. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('SYSTem:VNC:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, arg_0: bool) -> None:
		"""SCPI: SYSTem:VNC:STATe \n
		Snippet: driver.system.vnc.set_state(arg_0 = False) \n
		Enables or disables VNC state. \n
			:param arg_0:
				- 1: Enable VNC.
				- 0: Disable VNC."""
		param = Conversions.bool_to_str(arg_0)
		self._core.io.write(f'SYSTem:VNC:STATe {param}')

	def get_port(self) -> int:
		"""SCPI: SYSTem:VNC:PORT \n
		Snippet: value: int = driver.system.vnc.get_port() \n
		Sets or queries the VNC port number. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('SYSTem:VNC:PORT?')
		return Conversions.str_to_int(response)

	def set_port(self, arg_0: int) -> None:
		"""SCPI: SYSTem:VNC:PORT \n
		Snippet: driver.system.vnc.set_port(arg_0 = 1) \n
		Sets or queries the VNC port number. \n
			:param arg_0: No help available
		"""
		param = Conversions.decimal_value_to_str(arg_0)
		self._core.io.write(f'SYSTem:VNC:PORT {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Sequence:
	"""Sequence commands group definition. 6 total commands, 1 Sub-groups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("sequence", core, parent)

	@property
	def immediate(self):
		"""immediate commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_immediate'):
			from .Immediate import Immediate
			self._immediate = Immediate(self._core, self._cmd_group)
		return self._immediate



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Size:
	"""Size commands group definition. 2 total commands, 0 Sub-groups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("size", core, parent)

	def get_x(self) -> int:
		"""SCPI: HCOPy:SIZE:X \n
		Snippet: value: int = driver.hardCopy.size.get_x() \n
		Returns the horizontal dimension of the screenshots. \n
			:return: result: No help available
		"""
		response = self._core.io.query_str('HCOPy:SIZE:X?')
		return Conversions.str_to_int(response)

	def get_y(self) -> int:
		"""SCPI: HCOPy:SIZE:Y \n
		Snippet: value: int = driver.hardCopy.size.get_y() \n
		Returns the vertical dimension of the screenshots. \n
			:return: result: No help available
		"""
		response = self._core.io.query_str('HCOPy:SIZE:Y?')
		return Conversions.str_to_int(response)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from .....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Socket:
	"""Socket commands group definition. 7 total commands, 2 Sub-groups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("socket", core, parent)

	@property
	def apply(self):
		"""apply commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_apply'):
			from .Apply import Apply
			self._apply = Apply(self._core, self._cmd_group)
		return self._apply

	@property
	def discard(self):
		"""discard commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_discard'):
			from .Discard import Discard
			self._discard = Discard(self._core, self._cmd_group)
		return self._discard

	def get_dhcp(self) -> bool:
		"""SCPI: SYSTem:COMMunicate:SOCKet:DHCP \n
		Snippet: value: bool = driver.system.communicate.socket.get_dhcp() \n
		Sets the LAN interface mode. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:SOCKet:DHCP?')
		return Conversions.str_to_bool(response)

	def set_dhcp(self, arg_0: bool) -> None:
		"""SCPI: SYSTem:COMMunicate:SOCKet:DHCP \n
		Snippet: driver.system.communicate.socket.set_dhcp(arg_0 = False) \n
		Sets the LAN interface mode. \n
			:param arg_0:
				- 1: DHCP is enabled.Automatic IP address from DHCP server.
				- 0: DHCP is disabled.Manually set IP address."""
		param = Conversions.bool_to_str(arg_0)
		self._core.io.write(f'SYSTem:COMMunicate:SOCKet:DHCP {param}')

	def get_ip_address(self) -> str:
		"""SCPI: SYSTem:COMMunicate:SOCKet:IPADdress \n
		Snippet: value: str = driver.system.communicate.socket.get_ip_address() \n
		Sets or queries IP address of the LAN interface. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:SOCKet:IPADdress?')
		return trim_str_response(response)

	def set_ip_address(self, arg_0: str) -> None:
		"""SCPI: SYSTem:COMMunicate:SOCKet:IPADdress \n
		Snippet: driver.system.communicate.socket.set_ip_address(arg_0 = '1') \n
		Sets or queries IP address of the LAN interface. \n
			:param arg_0: IP address.
		"""
		param = Conversions.value_to_quoted_str(arg_0)
		self._core.io.write(f'SYSTem:COMMunicate:SOCKet:IPADdress {param}')

	def get_mask(self) -> str:
		"""SCPI: SYSTem:COMMunicate:SOCKet:MASK \n
		Snippet: value: str = driver.system.communicate.socket.get_mask() \n
		Sets or queries the subnet mask for LAN. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:SOCKet:MASK?')
		return trim_str_response(response)

	def set_mask(self, arg_0: str) -> None:
		"""SCPI: SYSTem:COMMunicate:SOCKet:MASK \n
		Snippet: driver.system.communicate.socket.set_mask(arg_0 = '1') \n
		Sets or queries the subnet mask for LAN. \n
			:param arg_0: Subnet address.
		"""
		param = Conversions.value_to_quoted_str(arg_0)
		self._core.io.write(f'SYSTem:COMMunicate:SOCKet:MASK {param}')

	def get_gateway(self) -> str:
		"""SCPI: SYSTem:COMMunicate:SOCKet:GATeway \n
		Snippet: value: str = driver.system.communicate.socket.get_gateway() \n
		Sets or queries gateway for LAN. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:SOCKet:GATeway?')
		return trim_str_response(response)

	def set_gateway(self, arg_0: str) -> None:
		"""SCPI: SYSTem:COMMunicate:SOCKet:GATeway \n
		Snippet: driver.system.communicate.socket.set_gateway(arg_0 = '1') \n
		Sets or queries gateway for LAN. \n
			:param arg_0: Gateway address.
		"""
		param = Conversions.value_to_quoted_str(arg_0)
		self._core.io.write(f'SYSTem:COMMunicate:SOCKet:GATeway {param}')

	def reset(self) -> None:
		"""SCPI: SYSTem:COMMunicate:SOCKet:RESet \n
		Snippet: driver.system.communicate.socket.reset() \n
		Resets LAN settings. \n
		"""
		self._core.io.write(f'SYSTem:COMMunicate:SOCKet:RESet')

	def reset_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: SYSTem:COMMunicate:SOCKet:RESet \n
		Snippet: driver.system.communicate.socket.reset_with_opc() \n
		Resets LAN settings. \n
		Same as reset, but waits for the operation to complete before continuing further. Use the RsNgx.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SYSTem:COMMunicate:SOCKet:RESet', opc_timeout_ms)



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Dio:
	"""Dio commands group definition. 7 total commands, 2 Sub-groups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("dio", core, parent)

	@property
	def fault(self):
		"""fault commands group. 0 Sub-classes, 4 commands."""
		if not hasattr(self, '_fault'):
			from .Fault import Fault
			self._fault = Fault(self._core, self._cmd_group)
		return self._fault

	@property
	def output(self):
		"""output commands group. 3 Sub-classes, 0 commands."""
		if not hasattr(self, '_output'):
			from .Output import Output
			self._output = Output(self._core, self._cmd_group)
		return self._output



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Key:
	"""Key commands group definition. 1 total commands, 0 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("key", core, parent)

	def get_brightness(self) -> float:
		"""SCPI: SYSTem:KEY:BRIGhtness \n
		Snippet: value: float = driver.system.key.get_brightness() \n
		Sets or queries the front panel key brightness. \n
			:return: front_key_brightness: No help available
		"""
		response = self._core.io.query_str('SYSTem:KEY:BRIGhtness?')
		return Conversions.str_to_float(response)

	def set_brightness(self, front_key_brightness: float) -> None:
		"""SCPI: SYSTem:KEY:BRIGhtness \n
		Snippet: driver.system.key.set_brightness(front_key_brightness = 1.0) \n
		Sets or queries the front panel key brightness. \n
			:param front_key_brightness: Sets the key brightness.
		"""
		param = Conversions.decimal_value_to_str(front_key_brightness)
		self._core.io.write(f'SYSTem:KEY:BRIGhtness {param}')



from typing import List

from .Internal.Core import Core
from .Internal.InstrumentErrors import RsInstrException
from .Internal.CommandsGroup import CommandsGroup
from .Internal.VisaSession import VisaSession
from .Internal.StructBase import StructBase
from .Internal.ArgStruct import ArgStruct


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class RsNgx:
	"""289 total commands, 21 Sub-groups, 1 group commands"""
	_driver_options = "SupportedInstrModels = NGP/NGL/NGM/NGU, SupportedIdnPatterns = NGP/NGL/NGM/NGU, SimulationIdnString = 'Rohde&Schwarz,NGP,100001,3.1.0.0049'"

	def __init__(self, resource_name: str, id_query: bool = True, reset: bool = False, options: str = None, direct_session: object = None):
		"""Initializes new RsNgx session. \n
		Parameter options tokens examples:
			- ``Simulate=True`` - starts the session in simulation mode. Default: ``False``
			- ``SelectVisa=socket`` - uses no VISA implementation for socket connections - you do not need any VISA-C installation
			- ``SelectVisa=rs`` - forces usage of RohdeSchwarz Visa
			- ``SelectVisa=ivi`` - forces usage of National Instruments Visa
			- ``QueryInstrumentStatus = False`` - same as ``driver.utilities.instrument_status_checking = False``. Default: ``True``
			- ``WriteDelay = 20, ReadDelay = 5`` - Introduces delay of 20ms before each write and 5ms before each read. Default: ``0ms`` for both
			- ``OpcWaitMode = OpcQuery`` - mode for all the opc-synchronised write/reads. Other modes: StbPolling, StbPollingSlow, StbPollingSuperSlow. Default: ``StbPolling``
			- ``AddTermCharToWriteBinBLock = True`` - Adds one additional LF to the end of the binary data (some instruments require that). Default: ``False``
			- ``AssureWriteWithTermChar = True`` - Makes sure each command/query is terminated with termination character. Default: Interface dependent
			- ``TerminationCharacter = "\\r"`` - Sets the termination character for reading. Default: ``\\n`` (LineFeed or LF)
			- ``DataChunkSize = 10E3`` - Maximum size of one write/read segment. If transferred data is bigger, it is split to more segments. Default: ``1E6`` bytes
			- ``OpcTimeout = 10000`` - same as driver.utilities.opc_timeout = 10000. Default: ``30000ms``
			- ``VisaTimeout = 5000`` - same as driver.utilities.visa_timeout = 5000. Default: ``10000ms``
			- ``ViClearExeMode = Disabled`` - viClear() execution mode. Default: ``execute_on_all``
			- ``OpcQueryAfterWrite = True`` - same as driver.utilities.opc_query_after_write = True. Default: ``False``
			- ``StbInErrorCheck = False`` - if true, the driver checks errors with *STB? If false, it uses SYST:ERR?. Default: ``True``
			- ``LoggingMode = LoggingMode.On`` - Sets the logging status right from the start. Default: ``Off``
			- ``LoggingName = 'MyDevice'`` - Sets the name to represent the session in the log entries. Default: ``'resource_name'``
		:param resource_name: VISA resource name, e.g. 'TCPIP::192.168.2.1::INSTR'
		:param id_query: if True: the instrument's model name is verified against the models supported by the driver and eventually throws an exception.
		:param reset: Resets the instrument (sends *RST command) and clears its status sybsystem
		:param options: string tokens alternating the driver settings.
		:param direct_session: Another driver object or pyVisa object to reuse the session instead of opening a new session."""
		self._core = Core(resource_name, id_query, reset, RsNgx._driver_options, options, direct_session)
		self._core.driver_version = '3.1.0.0049'
		self._options = options
		self._add_all_global_repcaps()
		self._custom_properties_init()
		self.utilities.default_instrument_setup()
		# noinspection PyTypeChecker
		self._cmd_group = CommandsGroup("ROOT", self._core, None)
		self._selected_channel = int(self._core.io.query_str('INSTrument:NSELect?', True))
		self._persistent_channel: int = 0

		def _before_write_callback(io, cmd: str) -> None:
			"""Callback that assures the INSTrument:NSELect value is persistent before every write command."""
			if self._persistent_channel == 0:
				return
			sent = cmd.split(' ')
			if sent[0] == 'INSTrument:NSELect':
				# We actually want to change the channel, can not do this with the persistence ON
				raise RsInstrException(f'You can not use the rsnrx.instrument.select.set() while the channel persistence is switched ON to channel {self._persistent_channel}.')
			if sent[0].startswith('APPLy'):
				# Exceptions for commands that do not need channel selection
				return
			self._assure_channel_selected(io)

		def _before_query_callback(io, cmd: str) -> None:
			"""Callback that assures the INSTrument:NSELect value is persistent before every query."""
			if self._persistent_channel == 0:
				return
			if cmd == 'INSTrument:NSELect?':
				# Exceptions for commands that do not need channel selection
				return
			self._assure_channel_selected(io)

		self._core.io.before_write_handler = _before_write_callback
		self._core.io.before_query_handler = _before_query_callback

	def set_persistent_channel(self, channel: int) -> None:
		"""If set to 0 (default value), the driver behaves exactly as the SCPI interface - you can select the addressed channel with rsnrx.instrument.select.set().
		If you set this property to 1 .. 8 (depending on how many channels your device has), the drivers makes sure that for each function you call,
		this channel is preselected, no matter which channel is currently active. In this mode you can not use the rsnrx.instrument.select.set()"""
		self._persistent_channel = channel
		if channel == 0:
			# For channel 0, do not do anything else
			return
		# Set the channel and make sure it will be persistent further on...
		self._core.io.write(f'INSTrument:NSELect {channel}', True)
		self._selected_channel = channel

	def get_persistent_channel(self) -> int:
		"""Returns the current persistent channel. If the persistence is switched off, the method returns 0."""
		return self._persistent_channel

	def _assure_channel_selected(self, io):
		# Make sure the persistent channel is selected
		status_check = io.query_instr_status
		try:
			io.query_instr_status = False
			channel = int(io.query_str('INSTrument:NSELect?', True))
			if channel != self._selected_channel:
				io.write(f'INSTrument:NSELect {self._selected_channel}', True)
		finally:
			io.query_instr_status = status_check

	@classmethod
	def from_existing_session(cls, session: object, options: str = None) -> 'RsNgx':
		"""Creates a new RsNgx object with the entered 'session' reused. \n
		:param session: can be an another driver or a direct pyvisa session.
		:param options: string tokens alternating the driver settings."""
		# noinspection PyTypeChecker
		return cls(None, False, False, options, session)

	def __str__(self) -> str:
		if self._core.io:
			return f"RsNgx session '{self._core.io.resource_name}'"
		else:
			return f"RsNgx with session closed"

	@staticmethod
	def assert_minimum_version(min_version: str) -> None:
		"""Asserts that the driver version fulfills the minimum required version you have entered.
		This way you make sure your installed driver is of the entered version or newer."""
		min_version_list = min_version.split('.')
		curr_version_list = '3.1.0.0049'.split('.')
		count_min = len(min_version_list)
		count_curr = len(curr_version_list)
		count = count_min if count_min < count_curr else count_curr
		for i in range(count):
			minimum = int(min_version_list[i])
			curr = int(curr_version_list[i])
			if curr > minimum:
				break
			if curr < minimum:
				raise RsInstrException(f"Assertion for minimum RsNgx version failed. Current version: '3.1.0.0049', minimum required version: '{min_version}'")
				
	@staticmethod
	def list_resources(expression: str = '?*::INSTR', visa_select: str = None) -> List[str]:
		"""Finds all the resources defined by the expression
			- '?*' - matches all the available instruments
			- 'USB::?*' - matches all the USB instruments
			- "TCPIP::192?*' - matches all the LAN instruments with the IP address starting with 192
		:param expression: see the examples in the function
		:param visa_select: optional parameter selecting a specific VISA. Examples: '@ivi', '@rs'
		"""
		rm = VisaSession.get_resource_manager(visa_select)
		resources = rm.list_resources(expression)
		rm.close()
		# noinspection PyTypeChecker
		return resources

	def close(self) -> None:
		"""Closes the active RsNgx session."""
		self._core.io.close()

	def get_session_handle(self) -> object:
		"""Returns the underlying session handle."""
		return self._core.get_session_handle()

	def _add_all_global_repcaps(self) -> None:
		"""Adds all the repcaps defined as global to the instrument's global repcaps dictionary."""

	def _custom_properties_init(self):
		"""Adds all the interfaces that are custom for the driver."""
		from .CustomFiles.utilities import Utilities
		self.utilities = Utilities(self._core)
		from .CustomFiles.events import Events
		self.events = Events(self._core)

	@property
	def log(self):
		"""log commands group. 7 Sub-classes, 4 commands."""
		if not hasattr(self, '_log'):
			from .Implementations.Log import Log
			self._log = Log(self._core, self._cmd_group)
		return self._log

	@property
	def apply(self):
		"""apply commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_apply'):
			from .Implementations.Apply import Apply
			self._apply = Apply(self._core, self._cmd_group)
		return self._apply

	@property
	def arbitrary(self):
		"""arbitrary commands group. 6 Sub-classes, 7 commands."""
		if not hasattr(self, '_arbitrary'):
			from .Implementations.Arbitrary import Arbitrary
			self._arbitrary = Arbitrary(self._core, self._cmd_group)
		return self._arbitrary

	@property
	def fuse(self):
		"""fuse commands group. 3 Sub-classes, 2 commands."""
		if not hasattr(self, '_fuse'):
			from .Implementations.Fuse import Fuse
			self._fuse = Fuse(self._core, self._cmd_group)
		return self._fuse

	@property
	def tracking(self):
		"""tracking commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_tracking'):
			from .Implementations.Tracking import Tracking
			self._tracking = Tracking(self._core, self._cmd_group)
		return self._tracking

	@property
	def calibration(self):
		"""calibration commands group. 7 Sub-classes, 7 commands."""
		if not hasattr(self, '_calibration'):
			from .Implementations.Calibration import Calibration
			self._calibration = Calibration(self._core, self._cmd_group)
		return self._calibration

	@property
	def output(self):
		"""output commands group. 6 Sub-classes, 3 commands."""
		if not hasattr(self, '_output'):
			from .Implementations.Output import Output
			self._output = Output(self._core, self._cmd_group)
		return self._output

	@property
	def system(self):
		"""system commands group. 12 Sub-classes, 1 commands."""
		if not hasattr(self, '_system'):
			from .Implementations.System import System
			self._system = System(self._core, self._cmd_group)
		return self._system

	@property
	def source(self):
		"""source commands group. 8 Sub-classes, 0 commands."""
		if not hasattr(self, '_source'):
			from .Implementations.Source import Source
			self._source = Source(self._core, self._cmd_group)
		return self._source

	@property
	def trigger(self):
		"""trigger commands group. 7 Sub-classes, 0 commands."""
		if not hasattr(self, '_trigger'):
			from .Implementations.Trigger import Trigger
			self._trigger = Trigger(self._core, self._cmd_group)
		return self._trigger

	@property
	def measure(self):
		"""measure commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_measure'):
			from .Implementations.Measure import Measure
			self._measure = Measure(self._core, self._cmd_group)
		return self._measure

	@property
	def sense(self):
		"""sense commands group. 3 Sub-classes, 0 commands."""
		if not hasattr(self, '_sense'):
			from .Implementations.Sense import Sense
			self._sense = Sense(self._core, self._cmd_group)
		return self._sense

	@property
	def dio(self):
		"""dio commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_dio'):
			from .Implementations.Dio import Dio
			self._dio = Dio(self._core, self._cmd_group)
		return self._dio

	@property
	def flog(self):
		"""flog commands group. 1 Sub-classes, 6 commands."""
		if not hasattr(self, '_flog'):
			from .Implementations.Flog import Flog
			self._flog = Flog(self._core, self._cmd_group)
		return self._flog

	@property
	def battery(self):
		"""battery commands group. 2 Sub-classes, 1 commands."""
		if not hasattr(self, '_battery'):
			from .Implementations.Battery import Battery
			self._battery = Battery(self._core, self._cmd_group)
		return self._battery

	@property
	def display(self):
		"""display commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_display'):
			from .Implementations.Display import Display
			self._display = Display(self._core, self._cmd_group)
		return self._display

	@property
	def hardCopy(self):
		"""hardCopy commands group. 2 Sub-classes, 1 commands."""
		if not hasattr(self, '_hardCopy'):
			from .Implementations.HardCopy import HardCopy
			self._hardCopy = HardCopy(self._core, self._cmd_group)
		return self._hardCopy

	@property
	def interfaces(self):
		"""interfaces commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_interfaces'):
			from .Implementations.Interfaces import Interfaces
			self._interfaces = Interfaces(self._core, self._cmd_group)
		return self._interfaces

	@property
	def data(self):
		"""data commands group. 2 Sub-classes, 2 commands."""
		if not hasattr(self, '_data'):
			from .Implementations.Data import Data
			self._data = Data(self._core, self._cmd_group)
		return self._data

	@property
	def instrument(self):
		"""instrument commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_instrument'):
			from .Implementations.Instrument import Instrument
			self._instrument = Instrument(self._core, self._cmd_group)
		return self._instrument

	@property
	def status(self):
		"""status commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_status'):
			from .Implementations.Status import Status
			self._status = Status(self._core, self._cmd_group)
		return self._status

	# noinspection PyTypeChecker
	class Measurement(StructBase):
		"""Response structure. Fields: \n
			- Voltage: float: No parameter help available
			- Current: float: No parameter help available"""
		__meta_args_list = [
			ArgStruct.scalar_float('Voltage'),
			ArgStruct.scalar_float('Current')]

		def __init__(self):
			StructBase.__init__(self, self)
			self.Voltage: float = None
			self.Current: float = None

	def read(self) -> Measurement:
		"""SCPI: READ \n
		Snippet: value: Measurement = driver.read() \n
		Queries for the next available readback for voltage and current of the selected channel. \n
			:return: structure: for return value, see the help for Measurement structure arguments."""
		return self._core.io.query_struct_with_opc(f'READ?', self.__class__.Measurement())

	def clone(self) -> 'RsNgx':
		"""Creates a deep copy of the RsNgx object.
		After cloning, your new RsNgx object can be used separately, but shares the same instrument session.
		Calling close() on the new object does not close the original VISA session.
		This method is similar to calling RsNgx.from_existing_session()"""
		cloned = RsNgx.from_existing_session(self.get_session_handle(), self._options)
		self._cmd_group.synchronize_repcaps(cloned)
		
		return cloned



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class State:
	"""State commands group definition. 1 total commands, 0 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("state", core, parent)

	def set(self, arg_0: bool) -> None:
		"""SCPI: TRIGger[:STATe] \n
		Snippet: driver.trigger.state.set(arg_0 = False) \n
		Enables or disables the trigger system. Upon being triggered, the selected trigger source method RsNgx.Trigger.Sequence.
		Immediate.Source.set becomes active. See Figure 'Overview of trigger IO system'. \n
			:param arg_0: 1 Enables the trigger system. 0 Disables the trigger system.
		"""
		param = Conversions.bool_to_str(arg_0)
		self._core.io.write(f'TRIGger:STATe {param}')

	def get(self) -> bool:
		"""SCPI: TRIGger[:STATe] \n
		Snippet: value: bool = driver.trigger.state.get() \n
		Enables or disables the trigger system. Upon being triggered, the selected trigger source method RsNgx.Trigger.Sequence.
		Immediate.Source.set becomes active. See Figure 'Overview of trigger IO system'. \n
			:return: arg_0: 1 Enables the trigger system. 0 Disables the trigger system."""
		response = self._core.io.query_str(f'TRIGger:STATe?')
		return Conversions.str_to_bool(response)



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Upper:
	"""Upper commands group definition. 1 total commands, 0 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("upper", core, parent)

	def set(self, current: float) -> None:
		"""SCPI: [SOURce]:CURRent[:LEVel][:IMMediate]:ALIMit[:UPPer] \n
		Snippet: driver.source.current.level.immediate.alimit.upper.set(current = 1.0) \n
		Sets or queries the upper safety limit for current. \n
			:param current:
				- numeric value: Numeric value for upper safety limit.
				- MIN | MINimum: Min value for upper safety limit.
				- MAX | MAXimum: Max value for upper safety limit."""
		param = Conversions.decimal_value_to_str(current)
		self._core.io.write(f'SOURce:CURRent:LEVel:IMMediate:ALIMit:UPPer {param}')

	def get(self) -> float:
		"""SCPI: [SOURce]:CURRent[:LEVel][:IMMediate]:ALIMit[:UPPer] \n
		Snippet: value: float = driver.source.current.level.immediate.alimit.upper.get() \n
		Sets or queries the upper safety limit for current. \n
			:return: result: No help available"""
		response = self._core.io.query_str(f'SOURce:CURRent:LEVel:IMMediate:ALIMit:UPPer?')
		return Conversions.str_to_float(response)



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Behavior:
	"""Behavior commands group definition. 1 total commands, 0 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("behavior", core, parent)

	# noinspection PyTypeChecker
	def get_end(self) -> enums.ArbEndBehavior:
		"""SCPI: ARBitrary:SEQuence:BEHavior:END \n
		Snippet: value: enums.ArbEndBehavior = driver.arbitrary.sequence.behavior.get_end() \n
		Sets or queries the arbitrary endpoint behavior, when QuickArb function is finished. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('ARBitrary:SEQuence:BEHavior:END?')
		return Conversions.str_to_scalar_enum(response, enums.ArbEndBehavior)

	def set_end(self, arg_0: enums.ArbEndBehavior) -> None:
		"""SCPI: ARBitrary:SEQuence:BEHavior:END \n
		Snippet: driver.arbitrary.sequence.behavior.set_end(arg_0 = enums.ArbEndBehavior.HOLD) \n
		Sets or queries the arbitrary endpoint behavior, when QuickArb function is finished. \n
			:param arg_0:
				- OFF: If the QuickArb function is finished, the respective channel is deactivated automatically.
				- HOLD: If the QuickArb function is finished, the last arbitrary point of the user-defined arbitrary list is held."""
		param = Conversions.enum_scalar_to_str(arg_0, enums.ArbEndBehavior)
		self._core.io.write(f'ARBitrary:SEQuence:BEHavior:END {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ...Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class File:
	"""File commands group definition. 2 total commands, 0 Sub-groups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("file", core, parent)

	def get_tpartition(self) -> str:
		"""SCPI: FLOG:FILE:TPARtition \n
		Snippet: value: str = driver.flog.file.get_tpartition() \n
		Selects the external partition to which the data is written into. \n
			:return: arg_0: Defines the external path partition to which the data is written in the USB stick.
		"""
		response = self._core.io.query_str('FLOG:FILE:TPARtition?')
		return trim_str_response(response)

	def set_tpartition(self, arg_0: str) -> None:
		"""SCPI: FLOG:FILE:TPARtition \n
		Snippet: driver.flog.file.set_tpartition(arg_0 = r1) \n
		Selects the external partition to which the data is written into. \n
			:param arg_0: Defines the external path partition to which the data is written in the USB stick.
		"""
		param = Conversions.value_to_str(arg_0)
		self._core.io.write(f'FLOG:FILE:TPARtition {param}')

	def get_duration(self) -> float:
		"""SCPI: FLOG:FILE:DURation \n
		Snippet: value: float = driver.flog.file.get_duration() \n
		Sets or queries the file write duration. \n
			:return: arg_0: No help available
		"""
		response = self._core.io.query_str('FLOG:FILE:DURation?')
		return Conversions.str_to_float(response)

	def set_duration(self, arg_0: float) -> None:
		"""SCPI: FLOG:FILE:DURation \n
		Snippet: driver.flog.file.set_duration(arg_0 = 1.0) \n
		Sets or queries the file write duration. \n
			:param arg_0: Sets file write duration.
		"""
		param = Conversions.decimal_value_to_str(arg_0)
		self._core.io.write(f'FLOG:FILE:DURation {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ...Internal.Types import DataType
from ...Internal.ArgSingleList import ArgSingleList
from ...Internal.ArgSingle import ArgSingle
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Interval:
	"""Interval commands group definition. 1 total commands, 0 Sub-groups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("interval", core, parent)

	def set(self, set_new_value: float, return_min_or_max: enums.MinOrMax = None) -> None:
		"""SCPI: LOG:INTerval \n
		Snippet: driver.log.interval.set(set_new_value = 1.0, return_min_or_max = enums.MinOrMax.MAX) \n
		Sets or queries the data logging measurement interval. The measurement interval describes the time between the recorded
		measurements. \n
			:param set_new_value:
				- numeric value: Measurement interval in the range of 0.008 s to 600 s.
				- MIN | MINimum: Minimum measurement interval is set at 0.008 s.
				- MAX | MAXimum: Maximum measurement interval is set at 600 s.
			:param return_min_or_max: Returns the measurement interval."""
		param = ArgSingleList().compose_cmd_string(ArgSingle('set_new_value', set_new_value, DataType.Float), ArgSingle('return_min_or_max', return_min_or_max, DataType.Enum, enums.MinOrMax, is_optional=True))
		self._core.io.write(f'LOG:INTerval {param}'.rstrip())

	def get(self, return_min_or_max: enums.MinOrMax = None) -> float:
		"""SCPI: LOG:INTerval \n
		Snippet: value: float = driver.log.interval.get(return_min_or_max = enums.MinOrMax.MAX) \n
		Sets or queries the data logging measurement interval. The measurement interval describes the time between the recorded
		measurements. \n
			:param return_min_or_max: Returns the measurement interval.
			:return: result: No help available"""
		param = ArgSingleList().compose_cmd_string(ArgSingle('return_min_or_max', return_min_or_max, DataType.Enum, enums.MinOrMax, is_optional=True))
		response = self._core.io.query_str(f'LOG:INTerval? {param}'.rstrip())
		return Conversions.str_to_float(response)



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Status:
	"""Status commands group definition. 20 total commands, 2 Sub-groups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("status", core, parent)

	@property
	def operation(self):
		"""operation commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_operation'):
			from .Operation import Operation
			self._operation = Operation(self._core, self._cmd_group)
		return self._operation

	@property
	def questionable(self):
		"""questionable commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_questionable'):
			from .Questionable import Questionable
			self._questionable = Questionable(self._core, self._cmd_group)
		return self._questionable
